@@ -6,7 +6,7 @@
 
 """defines the parse tree components for Mako templates."""
 
-from mako import exceptions, ast, util, filters
+from mako import exceptions, ast, util, filters, compat
 import re
 
 class Node(object):
@@ -20,8 +20,8 @@ class Node(object):
 
     @property
     def exception_kwargs(self):
-        return {'source':self.source, 'lineno':self.lineno,
-                'pos':self.pos, 'filename':self.filename}
+        return {'source': self.source, 'lineno': self.lineno,
+                'pos': self.pos, 'filename': self.filename}
 
     def get_children(self):
         return []
@@ -204,9 +204,9 @@ class _TagMeta(type):
     _classmap = {}
 
     def __init__(cls, clsname, bases, dict):
-        if cls.__keyword__ is not None:
+        if getattr(cls, '__keyword__', None) is not None:
             cls._classmap[cls.__keyword__] = cls
-            super(_TagMeta, cls).__init__(clsname, bases, dict)
+        super(_TagMeta, cls).__init__(clsname, bases, dict)
 
     def __call__(cls, keyword, attributes, **kwargs):
         if ":" in keyword:
@@ -226,7 +226,7 @@ class _TagMeta(type):
             )
         return type.__call__(cls, keyword, attributes, **kwargs)
 
-class Tag(Node):
+class Tag(compat.with_metaclass(_TagMeta, Node)):
     """abstract base class for tags.
 
     <%sometag/>
@@ -236,8 +236,6 @@ class Tag(Node):
     </%someothertag>
 
     """
-
-    __metaclass__ = _TagMeta
     __keyword__ = None
 
     def __init__(self, keyword, attributes, expressions,
@@ -547,7 +545,7 @@ class CallNamespaceTag(Tag):
                                 namespace,
                                 defname,
                                 ",".join(["%s=%s" % (k, v) for k, v in
-                                            self.parsed_attributes.iteritems()
+                                            self.parsed_attributes.items()
                                             if k != 'args'])
                             )
         self.code = ast.PythonCode(self.expression, **self.exception_kwargs)
