@@ -7,33 +7,36 @@
 """provides runtime services for templates, including Context,
 Namespace, and various helper functions."""
 
-from mako import exceptions, util
-import __builtin__, inspect, sys
+from mako import exceptions, util, compat
+from mako.compat import compat_builtins
+import inspect
+import sys
+import collections
 
 
 class Context(object):
     """Provides runtime namespace, output buffer, and various
     callstacks for templates.
- 
+
     See :ref:`runtime_toplevel` for detail on the usage of
     :class:`.Context`.
- 
+
      """
- 
+
     def __init__(self, buffer, **data):
         self._buffer_stack = [buffer]
- 
+
         self._data = data
 
         self._kwargs = data.copy()
         self._with_template = None
         self._outputting_as_unicode = None
         self.namespaces = {}
- 
-        # "capture" function which proxies to the 
+
+        # "capture" function which proxies to the
         # generic "capture" function
-        self._data['capture'] = util.partial(capture, self)
- 
+        self._data['capture'] = compat.partial(capture, self)
+
         # "caller" stack used by def calls with content
         self.caller_stack = self._data['caller'] = CallerStack()
 
@@ -42,91 +45,91 @@ class Context(object):
         illegal_names = t.reserved_names.intersection(self._data)
         if illegal_names:
             raise exceptions.NameConflictError(
-                "Reserved words passed to render(): %s" % 
+                "Reserved words passed to render(): %s" %
                 ", ".join(illegal_names))
 
     @property
     def lookup(self):
-        """Return the :class:`.TemplateLookup` associated 
+        """Return the :class:`.TemplateLookup` associated
         with this :class:`.Context`.
- 
+
         """
         return self._with_template.lookup
- 
+
     @property
     def kwargs(self):
-        """Return the dictionary of keyword argments associated with this
+        """Return the dictionary of keyword arguments associated with this
         :class:`.Context`.
- 
+
         """
         return self._kwargs.copy()
- 
+
     def push_caller(self, caller):
-        """Pushes a 'caller' callable onto the callstack for
+        """Push a ``caller`` callable onto the callstack for
         this :class:`.Context`."""
- 
- 
+
+
         self.caller_stack.append(caller)
- 
+
     def pop_caller(self):
-        """Pops a 'caller' callable onto the callstack for this
+        """Pop a ``caller`` callable onto the callstack for this
         :class:`.Context`."""
 
         del self.caller_stack[-1]
- 
+
     def keys(self):
         """Return a list of all names established in this :class:`.Context`."""
 
-        return self._data.keys()
- 
+        return list(self._data.keys())
+
     def __getitem__(self, key):
         if key in self._data:
             return self._data[key]
         else:
-            return __builtin__.__dict__[key]
+            return compat_builtins.__dict__[key]
 
     def _push_writer(self):
         """push a capturing buffer onto this Context and return
         the new writer function."""
- 
+
         buf = util.FastEncodingBuffer()
         self._buffer_stack.append(buf)
         return buf.write
 
     def _pop_buffer_and_writer(self):
-        """pop the most recent capturing buffer from this Context 
+        """pop the most recent capturing buffer from this Context
         and return the current writer after the pop.
- 
+
         """
 
         buf = self._buffer_stack.pop()
         return buf, self._buffer_stack[-1].write
- 
+
     def _push_buffer(self):
         """push a capturing buffer onto this Context."""
- 
+
         self._push_writer()
- 
+
     def _pop_buffer(self):
         """pop the most recent capturing buffer from this Context."""
- 
+
         return self._buffer_stack.pop()
- 
+
     def get(self, key, default=None):
         """Return a value from this :class:`.Context`."""
- 
-        return self._data.get(key, 
-                __builtin__.__dict__.get(key, default)
+
+        return self._data.get(key,
+                compat_builtins.__dict__.get(key, default)
                 )
- 
+
     def write(self, string):
         """Write a string to this :class:`.Context` object's
         underlying output buffer."""
- 
+
         self._buffer_stack[-1].write(string)
- 
+
     def writer(self):
-        """Return the current writer function"""
+        """Return the current writer function."""
 
         return self._buffer_stack[-1].write
 
@@ -140,17 +143,17 @@ class Context(object):
         c.namespaces = self.namespaces
         c.caller_stack = self.caller_stack
         return c
- 
+
     def locals_(self, d):
-        """create a new :class:`.Context` with a copy of this 
-        :class:`Context`'s current state, updated with the given dictionary."""
- 
+        """Create a new :class:`.Context` with a copy of this
+        :class:`.Context`'s current state, updated with the given dictionary."""
+
         if len(d) == 0:
             return self
         c = self._copy()
         c._data.update(d)
         return c
- 
+
     def _clean_inheritance_tokens(self):
         """create a new copy of this :class:`.Context`. with
         tokens related to inheritance state removed."""
@@ -165,8 +168,13 @@ class Context(object):
 class CallerStack(list):
     def __init__(self):
         self.nextcaller = None
+
     def __nonzero__(self):
+        return self.__bool__()
+
+    def __bool__(self):
         return self._get_caller() and True or False
+
     def _get_caller(self):
         # this method can be removed once
         # codegen MAGIC_NUMBER moves past 7
@@ -180,19 +188,23 @@ class CallerStack(list):
         return frame
     def _pop_frame(self):
         self.nextcaller = self.pop()
- 
- 
+
+
 class Undefined(object):
     """Represents an undefined value in a template.
- 
-    All template modules have a constant value 
+
+    All template modules have a constant value
     ``UNDEFINED`` present which is an instance of this
     object.
- 
+
     """
     def __str__(self):
         raise NameError("Undefined")
+
     def __nonzero__(self):
+        return self.__bool__()
+
+    def __bool__(self):
         return False
 
 UNDEFINED = Undefined()
@@ -239,24 +251,24 @@ class LoopStack(object):
 class LoopContext(object):
     """A magic loop variable.
     Automatically accessible in any ``% for`` block.
-    
+
     See the section :ref:`loop_context` for usage
     notes.
 
-    :attr:`parent` -> LoopContext or None
-        The parent loop, if one exists
-    :attr:`index` -> int
-        The 0-based iteration count
-    :attr:`reverse_index` -> int
-        The number of iterations remaining
-    :attr:`first` -> bool
-        `True` on the first iteration, `False` otherwise
-    :attr:`last` -> bool
-        `True` on the last iteration, `False` otherwise
-    :attr:`even` -> bool
-        `True` when `index` is even
-    :attr:`odd` -> bool
-        `True` when `index` is odd
+    :attr:`parent` -> :class:`.LoopContext` or ``None``
+        The parent loop, if one exists.
+    :attr:`index` -> `int`
+        The 0-based iteration count.
+    :attr:`reverse_index` -> `int`
+        The number of iterations remaining.
+    :attr:`first` -> `bool`
+        ``True`` on the first iteration, ``False`` otherwise.
+    :attr:`last` -> `bool`
+        ``True`` on the last iteration, ``False`` otherwise.
+    :attr:`even` -> `bool`
+        ``True`` when ``index`` is even.
+    :attr:`odd` -> `bool`
+        ``True`` when ``index`` is odd.
     """
 
     def __init__(self, iterable):
@@ -294,7 +306,7 @@ class LoopContext(object):
         return bool(self.index % 2)
 
     def cycle(self, *values):
-        """cycle through values as the loop progresses
+        """Cycle through values as the loop progresses.
         """
         if not values:
             raise ValueError("You must provide values to cycle through")
@@ -311,35 +323,37 @@ class _NSAttr(object):
                 return getattr(ns.module, key)
             else:
                 ns = ns.inherits
-        raise AttributeError(key) 
- 
+        raise AttributeError(key)
+
 class Namespace(object):
-    """Provides access to collections of rendering methods, which 
+    """Provides access to collections of rendering methods, which
       can be local, from other templates, or from imported modules.
- 
-      To access a particular rendering method referenced by a 
-      :class:`.Namespace`, use plain attribute access::
- 
+
+      To access a particular rendering method referenced by a
+      :class:`.Namespace`, use plain attribute access:
+
+      .. sourcecode:: mako
+
         ${some_namespace.foo(x, y, z)}
- 
-      :class:`.Namespace` also contains several built-in attributes 
+
+      :class:`.Namespace` also contains several built-in attributes
       described here.
- 
+
       """
- 
-    def __init__(self, name, context, 
-                            callables=None, inherits=None, 
+
+    def __init__(self, name, context,
+                            callables=None, inherits=None,
                             populate_self=True, calling_uri=None):
         self.name = name
         self.context = context
         self.inherits = inherits
         if callables is not None:
-            self.callables = dict([(c.func_name, c) for c in callables])
+            self.callables = dict([(c.__name__, c) for c in callables])
 
     callables = ()
 
     module = None
-    """The Python module referenced by this Namespace.
+    """The Python module referenced by this :class:`.Namespace`.
 
     If the namespace references a :class:`.Template`, then
     this module is the equivalent of ``template.module``,
@@ -354,8 +368,8 @@ class Namespace(object):
     """
 
     context = None
-    """The :class:`.Context` object for this namespace.
- 
+    """The :class:`.Context` object for this :class:`.Namespace`.
+
     Namespaces are often created with copies of contexts that
     contain slightly different data, particularly in inheritance
     scenarios. Using the :class:`.Context` off of a :class:`.Namespace` one
@@ -363,24 +377,24 @@ class Namespace(object):
     one-another.
 
     """
- 
+
     filename = None
     """The path of the filesystem file used for this
-    Namespace's module or template.
+    :class:`.Namespace`'s module or template.
 
     If this is a pure module-based
-    Namespace, this evaluates to ``module.__file__``. If a
+    :class:`.Namespace`, this evaluates to ``module.__file__``. If a
     template-based namespace, it evaluates to the original
     template file location.
 
     """
- 
+
     uri = None
-    """The uri for this Namespace's template.
+    """The URI for this :class:`.Namespace`'s template.
 
     I.e. whatever was sent to :meth:`.TemplateLookup.get_template()`.
 
-    This is the equivalent of :attr:`Template.uri`.
+    This is the equivalent of :attr:`.Template.uri`.
 
     """
 
@@ -388,83 +402,88 @@ class Namespace(object):
 
     @util.memoized_property
     def attr(self):
-        """Access module level attributes by name. 
- 
+        """Access module level attributes by name.
+
         This accessor allows templates to supply "scalar"
         attributes which are particularly handy in inheritance
-        relationships. See the example in
-        :ref:`inheritance_toplevel`.
+        relationships.
+
+        .. seealso::
+
+            :ref:`inheritance_attr`
+
+            :ref:`namespace_attr_for_includes`
 
         """
         return _NSAttr(self)
 
     def get_namespace(self, uri):
-        """Return a :class:`.Namespace` corresponding to the given uri.
- 
-        If the given uri is a relative uri (i.e. it does not
-        contain ia leading slash ``/``), the uri is adjusted to
-        be relative to the uri of the namespace itself. This
+        """Return a :class:`.Namespace` corresponding to the given ``uri``.
+
+        If the given ``uri`` is a relative URI (i.e. it does not
+        contain a leading slash ``/``), the ``uri`` is adjusted to
+        be relative to the ``uri`` of the namespace itself. This
         method is therefore mostly useful off of the built-in
-        ``local`` namespace, described in :ref:`namespace_local`
+        ``local`` namespace, described in :ref:`namespace_local`.
 
         In
         most cases, a template wouldn't need this function, and
         should instead use the ``<%namespace>`` tag to load
         namespaces. However, since all ``<%namespace>`` tags are
-        evaulated before the body of a template ever runs,
+        evaluated before the body of a template ever runs,
         this method can be used to locate namespaces using
         expressions that were generated within the body code of
         the template, or to conditionally use a particular
         namespace.
- 
+
         """
         key = (self, uri)
         if key in self.context.namespaces:
             return self.context.namespaces[key]
         else:
-            ns = TemplateNamespace(uri, self.context._copy(), 
-                                templateuri=uri, 
-                                calling_uri=self._templateuri) 
+            ns = TemplateNamespace(uri, self.context._copy(),
+                                templateuri=uri,
+                                calling_uri=self._templateuri)
             self.context.namespaces[key] = ns
             return ns
- 
+
     def get_template(self, uri):
-        """Return a :class:`.Template` from the given uri.
- 
-        The uri resolution is relative to the uri of this :class:`.Namespace`
+        """Return a :class:`.Template` from the given ``uri``.
+
+        The ``uri`` resolution is relative to the ``uri`` of this :class:`.Namespace`
         object's :class:`.Template`.
- 
+
         """
         return _lookup_template(self.context, uri, self._templateuri)
- 
+
     def get_cached(self, key, **kwargs):
-        """Return a value from the :class:`.Cache` referenced by this 
+        """Return a value from the :class:`.Cache` referenced by this
         :class:`.Namespace` object's :class:`.Template`.
- 
-        The advantage to this method versus direct access to the 
+
+        The advantage to this method versus direct access to the
         :class:`.Cache` is that the configuration parameters
         declared in ``<%page>`` take effect here, thereby calling
         up the same configured backend as that configured
         by ``<%page>``.
- 
+
         """
 
         return self.cache.get(key, **kwargs)
- 
+
     @property
     def cache(self):
-        """Return the :class:`.Cache` object referenced 
+        """Return the :class:`.Cache` object referenced
         by this :class:`.Namespace` object's
         :class:`.Template`.
- 
+
         """
         return self.template.cache
- 
+
     def include_file(self, uri, **kwargs):
-        """Include a file at the given uri"""
- 
+        """Include a file at the given ``uri``."""
+
         _include_file(self.context, uri, self._templateuri, **kwargs)
- 
+
     def _populate(self, d, l):
         for ident in l:
             if ident == '*':
@@ -472,7 +491,7 @@ class Namespace(object):
                     d[k] = v
             else:
                 d[ident] = getattr(self, ident)
- 
+
     def _get_star(self):
         if self.callables:
             for key in self.callables:
@@ -485,7 +504,7 @@ class Namespace(object):
             val = getattr(self.inherits, key)
         else:
             raise AttributeError(
-                    "Namespace '%s' has no member '%s'" % 
+                    "Namespace '%s' has no member '%s'" %
                     (self.name, key))
         setattr(self, key, val)
         return val
@@ -493,17 +512,17 @@ class Namespace(object):
 class TemplateNamespace(Namespace):
     """A :class:`.Namespace` specific to a :class:`.Template` instance."""
 
-    def __init__(self, name, context, template=None, templateuri=None, 
-                            callables=None, inherits=None, 
+    def __init__(self, name, context, template=None, templateuri=None,
+                            callables=None, inherits=None,
                             populate_self=True, calling_uri=None):
         self.name = name
         self.context = context
         self.inherits = inherits
         if callables is not None:
-            self.callables = dict([(c.func_name, c) for c in callables])
+            self.callables = dict([(c.__name__, c) for c in callables])
 
         if templateuri is not None:
-            self.template = _lookup_template(context, templateuri, 
+            self.template = _lookup_template(context, templateuri,
                                                 calling_uri)
             self._templateuri = self.template.module._template_uri
         elif template is not None:
@@ -514,13 +533,13 @@ class TemplateNamespace(Namespace):
 
         if populate_self:
             lclcallable, lclcontext = \
-                        _populate_self_namespace(context, self.template, 
+                        _populate_self_namespace(context, self.template,
                                                     self_ns=self)
 
     @property
     def module(self):
-        """The Python module referenced by this Namespace.
- 
+        """The Python module referenced by this :class:`.Namespace`.
+
         If the namespace references a :class:`.Template`, then
         this module is the equivalent of ``template.module``,
         i.e. the generated module for the template.
@@ -531,17 +550,17 @@ class TemplateNamespace(Namespace):
     @property
     def filename(self):
         """The path of the filesystem file used for this
-        Namespace's module or template.
+        :class:`.Namespace`'s module or template.
         """
         return self.template.filename
 
     @property
     def uri(self):
-        """The uri for this Namespace's template.
- 
+        """The URI for this :class:`.Namespace`'s template.
+
         I.e. whatever was sent to :meth:`.TemplateLookup.get_template()`.
- 
-        This is the equivalent of :attr:`Template.uri`.
+
+        This is the equivalent of :attr:`.Template.uri`.
 
         """
         return self.template.uri
@@ -552,7 +571,7 @@ class TemplateNamespace(Namespace):
                 yield (key, self.callables[key])
         def get(key):
             callable_ = self.template._get_def_callable(key)
-            return util.partial(callable_, self.context)
+            return compat.partial(callable_, self.context)
         for k in self.template.module._exports:
             yield (k, get(k))
 
@@ -561,13 +580,13 @@ class TemplateNamespace(Namespace):
             val = self.callables[key]
         elif self.template.has_def(key):
             callable_ = self.template._get_def_callable(key)
-            val = util.partial(callable_, self.context)
+            val = compat.partial(callable_, self.context)
         elif self.inherits:
             val = getattr(self.inherits, key)
 
         else:
             raise AttributeError(
-                    "Namespace '%s' has no member '%s'" % 
+                    "Namespace '%s' has no member '%s'" %
                     (self.name, key))
         setattr(self, key, val)
         return val
@@ -575,14 +594,14 @@ class TemplateNamespace(Namespace):
 class ModuleNamespace(Namespace):
     """A :class:`.Namespace` specific to a Python module instance."""
 
-    def __init__(self, name, context, module, 
-                            callables=None, inherits=None, 
+    def __init__(self, name, context, module,
+                            callables=None, inherits=None,
                             populate_self=True, calling_uri=None):
         self.name = name
         self.context = context
         self.inherits = inherits
         if callables is not None:
-            self.callables = dict([(c.func_name, c) for c in callables])
+            self.callables = dict([(c.__name__, c) for c in callables])
 
         mod = __import__(module)
         for token in module.split('.')[1:]:
@@ -592,7 +611,7 @@ class ModuleNamespace(Namespace):
     @property
     def filename(self):
         """The path of the filesystem file used for this
-        Namespace's module or template.
+        :class:`.Namespace`'s module or template.
         """
         return self.module.__file__
 
@@ -602,7 +621,7 @@ class ModuleNamespace(Namespace):
                 yield (key, self.callables[key])
         def get(key):
             callable_ = getattr(self.module, key)
-            return util.partial(callable_, self.context)
+            return compat.partial(callable_, self.context)
         for k in dir(self.module):
             if k[0] != '_':
                 yield (k, get(k))
@@ -612,12 +631,12 @@ class ModuleNamespace(Namespace):
             val = self.callables[key]
         elif hasattr(self.module, key):
             callable_ = getattr(self.module, key)
-            val = util.partial(callable_, self.context)
+            val = compat.partial(callable_, self.context)
         elif self.inherits:
             val = getattr(self.inherits, key)
         else:
             raise AttributeError(
-                    "Namespace '%s' has no member '%s'" % 
+                    "Namespace '%s' has no member '%s'" %
                     (self.name, key))
         setattr(self, key, val)
         return val
@@ -625,11 +644,11 @@ class ModuleNamespace(Namespace):
 def supports_caller(func):
     """Apply a caller_stack compatibility decorator to a plain
     Python function.
- 
+
     See the example in :ref:`namespaces_python_modules`.
- 
+
     """
- 
+
     def wrap_stackframe(context,  *args, **kwargs):
         context.caller_stack._push_frame()
         try:
@@ -637,16 +656,16 @@ def supports_caller(func):
         finally:
             context.caller_stack._pop_frame()
     return wrap_stackframe
- 
+
 def capture(context, callable_, *args, **kwargs):
     """Execute the given template def, capturing the output into
     a buffer.
- 
+
     See the example in :ref:`namespaces_python_modules`.
- 
+
     """
- 
-    if not callable(callable_):
+
+    if not compat.callable(callable_):
         raise exceptions.RuntimeException(
                            "capture() function expects a callable as "
                            "its argument (i.e. capture(func, *args, **kwargs))"
@@ -671,7 +690,7 @@ def _decorate_toplevel(fn):
             return fn(y)(context, *args, **kw)
         return go
     return decorate_render
- 
+
 def _decorate_inline(context, fn):
     def decorate_render(render_fn):
         dec = fn(render_fn)
@@ -679,17 +698,17 @@ def _decorate_inline(context, fn):
             return dec(context, *args, **kw)
         return go
     return decorate_render
- 
+
 def _include_file(context, uri, calling_uri, **kwargs):
     """locate the template from the given uri and include it in
     the current output."""
- 
+
     template = _lookup_template(context, uri, calling_uri)
     (callable_, ctx) = _populate_self_namespace(
-                                context._clean_inheritance_tokens(), 
+                                context._clean_inheritance_tokens(),
                                 template)
     callable_(ctx, **_kwargs_for_include(callable_, context._data, **kwargs))
- 
+
 def _inherit_from(context, uri, calling_uri):
     """called by the _inherit method in template modules to set
     up the inheritance chain at the start of a template's
@@ -703,9 +722,9 @@ def _inherit_from(context, uri, calling_uri):
     while ih.inherits is not None:
         ih = ih.inherits
     lclcontext = context.locals_({'next':ih})
-    ih.inherits = TemplateNamespace("self:%s" % template.uri, 
-                                lclcontext, 
-                                template = template, 
+    ih.inherits = TemplateNamespace("self:%s" % template.uri,
+                                lclcontext,
+                                template = template,
                                 populate_self=False)
     context._data['parent'] = lclcontext._data['local'] = ih.inherits
     callable_ = getattr(template.module, '_mako_inherit', None)
@@ -723,18 +742,18 @@ def _lookup_template(context, uri, relativeto):
     lookup = context._with_template.lookup
     if lookup is None:
         raise exceptions.TemplateLookupException(
-                            "Template '%s' has no TemplateLookup associated" % 
+                            "Template '%s' has no TemplateLookup associated" %
                             context._with_template.uri)
     uri = lookup.adjust_uri(uri, relativeto)
     try:
         return lookup.get_template(uri)
-    except exceptions.TopLevelLookupException, e:
-        raise exceptions.TemplateLookupException(str(e))
+    except exceptions.TopLevelLookupException:
+        raise exceptions.TemplateLookupException(str(compat.exception_as()))
 
 def _populate_self_namespace(context, template, self_ns=None):
     if self_ns is None:
-        self_ns = TemplateNamespace('self:%s' % template.uri, 
-                                context, template=template, 
+        self_ns = TemplateNamespace('self:%s' % template.uri,
+                                context, template=template,
                                 populate_self=False)
     context._data['self'] = context._data['local'] = self_ns
     if hasattr(template.module, '_mako_inherit'):
@@ -744,32 +763,32 @@ def _populate_self_namespace(context, template, self_ns=None):
     return (template.callable_, context)
 
 def _render(template, callable_, args, data, as_unicode=False):
-    """create a Context and return the string 
+    """create a Context and return the string
     output of the given template and template callable."""
 
     if as_unicode:
-        buf = util.FastEncodingBuffer(unicode=True)
+        buf = util.FastEncodingBuffer(as_unicode=True)
     elif template.bytestring_passthrough:
-        buf = util.StringIO()
+        buf = compat.StringIO()
     else:
         buf = util.FastEncodingBuffer(
-                        unicode=as_unicode, 
-                        encoding=template.output_encoding, 
+                        as_unicode=as_unicode,
+                        encoding=template.output_encoding,
                         errors=template.encoding_errors)
     context = Context(buf, **data)
     context._outputting_as_unicode = as_unicode
     context._set_with_template(template)
- 
-    _render_context(template, callable_, context, *args, 
+
+    _render_context(template, callable_, context, *args,
                             **_kwargs_for_callable(callable_, data))
     return context._pop_buffer().getvalue()
 
 def _kwargs_for_callable(callable_, data):
-    argspec = util.inspect_func_args(callable_)
+    argspec = compat.inspect_func_args(callable_)
     # for normal pages, **pageargs is usually present
     if argspec[2]:
         return data
- 
+
     # for rendering defs from the top level, figure out the args
     namedargs = argspec[0] + [v for v in argspec[1:3] if v is not None]
     kwargs = {}
@@ -779,16 +798,16 @@ def _kwargs_for_callable(callable_, data):
     return kwargs
 
 def _kwargs_for_include(callable_, data, **kwargs):
-    argspec = util.inspect_func_args(callable_)
+    argspec = compat.inspect_func_args(callable_)
     namedargs = argspec[0] + [v for v in argspec[1:3] if v is not None]
     for arg in namedargs:
         if arg != 'context' and arg in data and arg not in kwargs:
             kwargs[arg] = data[arg]
     return kwargs
- 
+
 def _render_context(tmpl, callable_, context, *args, **kwargs):
     import mako.template as template
-    # create polymorphic 'self' namespace for this 
+    # create polymorphic 'self' namespace for this
     # template with possibly updated context
     if not isinstance(tmpl, template.DefTemplate):
         # if main render method, call from the base of the inheritance stack
@@ -798,7 +817,7 @@ def _render_context(tmpl, callable_, context, *args, **kwargs):
         # otherwise, call the actual rendering method specified
         (inherit, lclcontext) = _populate_self_namespace(context, tmpl.parent)
         _exec_template(callable_, context, args=args, kwargs=kwargs)
- 
+
 def _exec_template(callable_, context, args=None, kwargs=None):
     """execute a rendering callable given the callable, a
     Context, and optional explicit arguments
@@ -813,9 +832,9 @@ def _exec_template(callable_, context, args=None, kwargs=None):
         error = None
         try:
             callable_(context, *args, **kwargs)
-        except Exception, e:
-            _render_error(template, context, e)
-        except: 
+        except Exception:
+            _render_error(template, context, compat.exception_as())
+        except:
             e = sys.exc_info()[0]
             _render_error(template, context, e)
     else:
@@ -829,11 +848,11 @@ def _render_error(template, context, error):
     else:
         error_template = exceptions.html_error_template()
         if context._outputting_as_unicode:
-            context._buffer_stack[:] = [util.FastEncodingBuffer(unicode=True)]
+            context._buffer_stack[:] = [util.FastEncodingBuffer(as_unicode=True)]
         else:
             context._buffer_stack[:] = [util.FastEncodingBuffer(
                                             error_template.output_encoding,
                                             error_template.encoding_errors)]
- 
+
         context._set_with_template(error_template)
         error_template.render_context(context, error=error)
