@@ -1,16 +1,15 @@
 # -*- coding: utf-8 -*-
 import sys
-import unittest
 
-from mako import exceptions, util
+from mako import exceptions, compat
 from mako.template import Template
 from mako.lookup import TemplateLookup
-from util import result_lines
-from test import template_base, module_base, TemplateTest
-from test import requires_pygments_14, requires_no_pygments, \
+from mako.compat import u
+from test.util import result_lines
+from test import TemplateTest
+from test import requires_pygments_14, requires_no_pygments_exceptions, \
     requires_python_25_or_greater
 
-
 class ExceptionsTest(TemplateTest):
     def test_html_error_template(self):
         """test the html_error_template"""
@@ -21,7 +20,7 @@ class ExceptionsTest(TemplateTest):
             template = Template(code)
             template.render_unicode()
             assert False
-        except exceptions.CompileException, ce:
+        except exceptions.CompileException:
             html_error = exceptions.html_error_template().render_unicode()
             assert ("CompileException: Fragment &#39;i = 0&#39; is not "
                     "a partial control statement at line: 2 char: 1") in html_error
@@ -50,7 +49,7 @@ class ExceptionsTest(TemplateTest):
             template = Template(code)
             template.render_unicode()
             assert False
-        except exceptions.CompileException, ce:
+        except exceptions.CompileException:
             text_error = exceptions.text_error_template().render_unicode()
             assert 'Traceback (most recent call last):' in text_error
             assert ("CompileException: Fragment 'i = 0' is not a partial "
@@ -61,7 +60,7 @@ class ExceptionsTest(TemplateTest):
         """test the html_error_template with a Template containing utf8
         chars"""
 
-        if util.py3k:
+        if compat.py3k:
             code = """# -*- coding: utf-8 -*-
 % if 2 == 2: /an error
 ${'привет'}
@@ -76,9 +75,9 @@ ${u'привет'}
         try:
             template = Template(code)
             template.render_unicode()
-        except exceptions.CompileException, ce:
+        except exceptions.CompileException:
             html_error = exceptions.html_error_template().render()
-            if util.py3k:
+            if compat.py3k:
                 assert ("CompileException: Fragment &#39;if 2 == 2: /an "
                     "error&#39; is not a partial control statement "
                     "at line: 2 char: 1").encode(sys.getdefaultencoding(), 'htmlentityreplace') in \
@@ -89,11 +88,11 @@ ${u'привет'}
                         "at line: 2 char: 1") in \
                         html_error
 
-            if util.py3k:
-                assert u"".encode(sys.getdefaultencoding(),
+            if compat.py3k:
+                assert "".encode(sys.getdefaultencoding(),
                                         'htmlentityreplace') in html_error
             else:
-                assert u'<pre>3</pre></div></td><td class="code">'\
+                assert '<pre>3</pre></div></td><td class="code">'\
                         '<div class="syntax-highlighted"><pre><span '\
                         'class="cp">${</span><span class="s">u&#39;'\
                         '&#x43F;&#x440;&#x438;&#x432;&#x435;&#x442;'\
@@ -104,12 +103,12 @@ ${u'привет'}
             assert False, ("This function should trigger a CompileException, "
                            "but didn't")
 
-    @requires_no_pygments
+    @requires_no_pygments_exceptions
     def test_utf8_html_error_template_no_pygments(self):
         """test the html_error_template with a Template containing utf8
         chars"""
 
-        if util.py3k:
+        if compat.py3k:
             code = """# -*- coding: utf-8 -*-
 % if 2 == 2: /an error
 ${'привет'}
@@ -124,12 +123,13 @@ ${u'привет'}
         try:
             template = Template(code)
             template.render_unicode()
-        except exceptions.CompileException, ce:
+        except exceptions.CompileException:
             html_error = exceptions.html_error_template().render()
-            if util.py3k:
+            if compat.py3k:
                 assert ("CompileException: Fragment &#39;if 2 == 2: /an "
                     "error&#39; is not a partial control statement "
-                    "at line: 2 char: 1").encode(sys.getdefaultencoding(), 'htmlentityreplace') in \
+                    "at line: 2 char: 1").encode(sys.getdefaultencoding(),
+                            'htmlentityreplace') in \
                     html_error
             else:
                 assert ("CompileException: Fragment &#39;if 2 == 2: /an "
@@ -137,21 +137,20 @@ ${u'привет'}
                         "at line: 2 char: 1") in \
                         html_error
 
-            if util.py3k:
-                assert u"${&#39;привет&#39;}".encode(sys.getdefaultencoding(),
-                                        'htmlentityreplace') in html_error
+            if compat.py3k:
+                assert "${&#39;привет&#39;}".encode(sys.getdefaultencoding(),
+                                 'htmlentityreplace') in html_error
             else:
-                assert u"${u&#39;привет&#39;}".encode(sys.getdefaultencoding(),
-                                        'htmlentityreplace') in html_error
+                assert u("${u&#39;привет&#39;}").encode(sys.getdefaultencoding(),
+                                    'htmlentityreplace') in html_error
         else:
             assert False, ("This function should trigger a CompileException, "
                            "but didn't")
 
     def test_format_closures(self):
         try:
-            exec "def foo():"\
-                 "    raise RuntimeError('test')"\
-                 in locals()
+            exec("def foo():"\
+                 "    raise RuntimeError('test')", locals())
             foo()
         except:
             html_error = exceptions.html_error_template().render()
@@ -160,23 +159,23 @@ ${u'привет'}
     @requires_python_25_or_greater
     def test_py_utf8_html_error_template(self):
         try:
-            foo = u'日本'
+            foo = u('日本')
             raise RuntimeError('test')
         except:
             html_error = exceptions.html_error_template().render()
-            if util.py3k:
+            if compat.py3k:
                 assert 'RuntimeError: test' in html_error.decode('utf-8')
-                assert u"foo = &#39;日本&#39;" in html_error.decode('utf-8')
+                assert "foo = u(&#39;日本&#39;)" in html_error.decode('utf-8')
             else:
                 assert 'RuntimeError: test' in html_error
-                assert "foo = u&#39;&#x65E5;&#x672C;&#39;" in html_error
+                assert "foo = u(&#39;&#x65E5;&#x672C;&#39;)" in html_error
 
     def test_py_unicode_error_html_error_template(self):
         try:
-            raise RuntimeError(u'日本')
+            raise RuntimeError(u('日本'))
         except:
             html_error = exceptions.html_error_template().render()
-            assert u"RuntimeError: 日本".encode('ascii', 'ignore') in html_error
+            assert u("RuntimeError: 日本").encode('ascii', 'ignore') in html_error
 
     @requires_pygments_14
     def test_format_exceptions_pygments(self):
@@ -198,7 +197,7 @@ ${foobar}
                 '<span class="n">foobar</span><span class="p">}</span>' in \
             result_lines(l.get_template("foo.html").render_unicode())
 
-    @requires_no_pygments
+    @requires_no_pygments_exceptions
     def test_format_exceptions_no_pygments(self):
         l = TemplateLookup(format_exceptions=True)
 
@@ -220,12 +219,12 @@ ${foobar}
            exceptions reported with format_exceptions=True"""
 
         l = TemplateLookup(format_exceptions=True)
-        if util.py3k:
+        if compat.py3k:
             l.put_string("foo.html", """# -*- coding: utf-8 -*-\n${'привет' + foobar}""")
         else:
             l.put_string("foo.html", """# -*- coding: utf-8 -*-\n${u'привет' + foobar}""")
 
-        if util.py3k:
+        if compat.py3k:
             assert '<table class="error syntax-highlightedtable"><tr><td '\
                     'class="linenos"><div class="linenodiv"><pre>2</pre>'\
                     '</div></td><td class="code"><div class="error '\
@@ -245,19 +244,19 @@ ${foobar}
                     '<span class="x"></span>' in \
                 result_lines(l.get_template("foo.html").render().decode('utf-8'))
 
-    @requires_no_pygments
+    @requires_no_pygments_exceptions
     def test_utf8_format_exceptions_no_pygments(self):
         """test that htmlentityreplace formatting is applied to
            exceptions reported with format_exceptions=True"""
 
         l = TemplateLookup(format_exceptions=True)
-        if util.py3k:
+        if compat.py3k:
             l.put_string("foo.html", """# -*- coding: utf-8 -*-\n${'привет' + foobar}""")
         else:
             l.put_string("foo.html", """# -*- coding: utf-8 -*-\n${u'привет' + foobar}""")
 
-        if util.py3k:
-            assert u'<div class="sourceline">${&#39;привет&#39; + foobar}</div>'\
+        if compat.py3k:
+            assert '<div class="sourceline">${&#39;привет&#39; + foobar}</div>'\
                 in result_lines(l.get_template("foo.html").render().decode('utf-8'))
         else:
             assert '${u&#39;&#x43F;&#x440;&#x438;&#x432;&#x435;'\
@@ -283,14 +282,14 @@ ${foobar}
         # isn't in the 'wrong' exception
         assert "".join(reversed(");93#&rab;93#&(oof")) in html_error
 
-    def test_tback_no_trace(self):
+    def test_tback_no_trace_from_py_file(self):
         try:
             t = self._file_template("runtimeerr.html")
             t.render()
         except:
             t, v, tback = sys.exc_info()
 
-        if not util.py3k:
+        if not compat.py3k:
             # blow away tracebaack info
             sys.exc_clear()
 
@@ -298,3 +297,14 @@ ${foobar}
         html_error = exceptions.html_error_template().\
                     render_unicode(error=v, traceback=None)
         assert "local variable &#39;y&#39; referenced before assignment" in html_error
+
+    def test_tback_trace_from_py_file(self):
+        t = self._file_template("runtimeerr.html")
+        try:
+            t.render()
+            assert False
+        except:
+            html_error = exceptions.html_error_template().\
+                        render_unicode()
+
+        assert "local variable &#39;y&#39; referenced before assignment" in html_error
